#ifndef TIMEPIX_OCCUPANCY_PROCESSOR_H
#define TIMEPIX_OCCUPANCY_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include <marlin/Processor.h>

//C++
#include <map>


namespace AIDA{
  class IHistogram2D;
}


namespace marlintpc
{

    class TrackFitterFactory;
//    class TrackFitterBase;

    /** Count how many times a pixel has been hit on the TimePix chip
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li A Collection of TimePixRawData or TimePixZeroSuppressedRawData
     *
     *  <h4>Output</h4>
     *  \li One 2D Histogram with the number of hits per pixel for each chip
     *
     *  @param InputColName Name of input collection with lcio::TrackerRawData (default: TimePixRawData)
     *  @param NColumns Number of colmns on a chip (default: 256)
     *  @param NRows Number of rows on a chip (default: 256)
     *
     *  @author Martin Killenberg, University of Bonn
     */

    class TimePixOccupancyProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new TimePixOccupancyProcessor ; }
    
	TimePixOccupancyProcessor();

	~TimePixOccupancyProcessor();

	virtual void init();

	virtual void processRunHeader( EVENT::LCRunHeader* run ) ;

	virtual void processEvent( LCEvent * evt ) ; 
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     
//     TrackFitterBase *trackFitter;
//     TrackFitterFactory *_fitterFactory; ///< The fitter factory

     ///< Map with the occupancy histograms, one per chip
     std::map<int,     AIDA::IHistogram2D *>  _timePixOccupancyHistos;

     int  _nColumns; ///< Number of colmns on the chip
     int  _nRows; ///< Number of rows on the chip

    };
}

#endif // TIMEPIX_OCCUPANCY_PROCESSOR_H
