#ifndef XYZ_DISTRIBUTION_PROCESSOR_H
#define XYZ_DISTRIBUTION_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include "marlin/Processor.h"

namespace AIDA{
  class IHistogram1D;
  class IHistogram2D;
}


namespace marlintpc
{

    class TrackFitterFactory;
//    class TrackFitterBase;

    /** Fill the x, y, and z positions of all hits into an AIDA histogram.
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li A Collection of reconstructed hits
     *
     *  <h4>Output</h4>
     *  \li Histograms of the x, y and z position distributions
     *
     *  @param InputTrackes Name of input Hits collection (default: TPCHits)
     *  @param HistogramMinValueX  Minimum value in x histogram (default: 0. mm)
     *  @param HistogramMaxValueX  Maximal value in x histogram (default: 15. mm)
     *  @param HistogramMinValueY  Minimum value in y histogram (default: 0. mm)
     *  @param HistogramMaxValueY  Maximal value in y histogram (default: 15. mm)
     *  @param HistogramMinValueZ  Minimum value in z histogram (default: 0.0 mm)
     *  @param HistogramMaxValueZ  Maximal value in z histogram (default: 260.0 mm)
     *
     *
     *  @author Martin Killenberg, University of Bonn
     */
    class XYZDistributionProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new XYZDistributionProcessor ; }

	XYZDistributionProcessor();
    
	~XYZDistributionProcessor();

	virtual void init();

	virtual void processRunHeader( EVENT::LCRunHeader* run ) ;

	virtual void processEvent( LCEvent * evt ) ; 
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     
//     TrackFitterBase *trackFitter;
//     TrackFitterFactory *_fitterFactory; ///< The fitter factory

     AIDA::IHistogram1D * _distributionXHisto; ///< The histogram of the x values
     AIDA::IHistogram1D * _distributionYHisto; ///< The histogram of the y values
     AIDA::IHistogram1D * _distributionZHisto; ///< The histogram of the z values
     AIDA::IHistogram2D * _distributionXYHisto; ///< The histogram of the xy values
     float  _histogramMinValueX; ///< Minimum value in x histogram
     float  _histogramMaxValueX; ///< Maximum value in x histogram
     float  _histogramMinValueY; ///< Minimum value in y histogram
     float  _histogramMaxValueY; ///< Maximum value in y histogram

     float  _histogramMinValueZ; ///< Minimum value in z histogram
     float  _histogramMaxValueZ; ///< Maximum value in z histogram

    };

}

#endif // XYZ_DISTRIBUTION_PROCESSOR_H
