#ifdef USE_LCCD

#ifndef PHOTOELECTRICLIKELIHOODFITTINGPROCESSOR_H
#define PHOTOELECTRICLIKELIHOODFITTINGPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

// GEAR
#include <gear/TPCParameters.h>
#include <gearimpl/GlobalPadIndex.h>

// LCIO
#include <EVENT/TrackerHit.h>

/* NOTE this uses the C++ version of Minuit - not the Fortran version */
#include <Minuit2/FCNBase.h>

// CLHEP
#include <CLHEP/Vector/TwoVector.h>

// stl
#include <string>
#include <set>
#include <list>

namespace EVENT {

	class TrackerPulse;
}

namespace AIDA {

	class ITuple;
}

namespace marlintpc {

/** PhotoelectricLikelihoodFittingProcessor: performs a likelihood fit on photoelectric data.
 *    @author J. Abernathy (UVic)<br>
 */
class PhotoelectricLikelihoodFittingProcessor : public marlin::Processor {

public:

	PhotoelectricLikelihoodFittingProcessor() ;

	virtual void init() ;

	virtual Processor*  newProcessor() { return new PhotoelectricLikelihoodFittingProcessor(); }

	virtual void processRunHeader( lcio::LCRunHeader* run );

	virtual void processEvent( lcio::LCEvent * evt );

	virtual void check( lcio::LCEvent * evt );

	virtual void end();

private:

	/** the name of the input collection of TPCHits which represent photodots
	*/
	std::string _input_photoelectric_tracker_pulses_collection_name;

	/** the name of the output collection of TPCHits which represent fit photodots
	 */
	std::string _output_photodot_collection_name;

	/** the name of the output collection of Tracks which represent fit photolines
	 */
	std::string _output_photoline_collection_name;

	/** determines whether the output of this processor will be stored in a file after program termination (false) or not (true - default)
	*/
	bool _output_is_transient;

	/** specifies the name of the collection which stores the results of the projection finding
	 */
	std::string _projected_photogeometry_file_name;

	/**
	 */
	double _sigma;

	/** this tuple holds the results of each photodot fit
	 */
	AIDA::ITuple *_photodot_fit_results_tuple;

	/** this map contains a pointer to the set of TrackerPulses for each photodot
	 */
	std::map< gear::GlobalPadIndex, std::list< std::set< EVENT::TrackerPulse* > >::iterator > _photodot_pulse_set_by_global_index;

	/** the actual collection of sets of pulses, one for each photodot
	 */
	std::list< std::set< EVENT::TrackerPulse* > > _photodot_pulse_sets;

	/** the centroids of each photodot
	 */
	std::list< CLHEP::Hep2Vector > _photodot_centroids;

	/** this map contains a pointer to the set of TrackerPulses for each photoline
	 */
	std::map< gear::GlobalPadIndex, std::list< std::set< EVENT::TrackerPulse* > >::iterator > _photoline_pulse_set_by_global_index;

	/** the actual collection of sets of pulses, one for each photoline
	 */
	std::list< std::set< EVENT::TrackerPulse* > > _photoline_pulse_sets;

	/** this helper class is used to calculate the likelihood function while fitting photodots
	 */
	class PhotodotLikelihood : public ROOT::Minuit2::FCNBase {

	public:

		PhotodotLikelihood( const std::set< EVENT::TrackerPulse* > &photodot_data );

		virtual double operator()(const std::vector<double>& x) const;

		virtual double Up() const;

	private:

		const std::set< EVENT::TrackerPulse* > &_photodot_data;
	};

};

}

#endif // PHOTOELECTRICLIKELIHOODFITTINGPROCESSOR_H
#endif // USE_LCCD
