#include "MappingHandler.h"
#include <iostream>
#include <algorithm>
#include <sstream>

#include <IMPL/LCCollectionVec.h>
using namespace lcio;
//using namespace marlin;
using namespace std;
using namespace tpcconddata;


namespace marlintpc{

MappingHandler::MappingHandler()
  : chanMapSortedHardwareID("ADCChannelMapping") ,
    chanMapSortedSoftwareID("ADCChannelMapping")
{
  // constructor is empty, but needed for the
  // constructor initializer list
  // i. e. chanMapSortedChanID and chanMapSortedPadID
  // have to be initialised with a string
}

void MappingHandler:: setChannelMap(LCCollection *chanMapCol)
{  // copy the channel map to a local copy which 
  // is then sortet according to channelID
  // cout << "This is  MappingHandler:: setChannelMap" << endl; // debug
  chanMapSortedHardwareID=*(dynamic_cast<LCCollectionVec *>(chanMapCol));

  // ok, we did what we should not have to, i.e. calling the copy constructor
  // at least we can set the subset flag so the objects which are not owned by this
  // collection are not deleted in the destructor
  chanMapSortedHardwareID.setSubset();
  // cout << "length of vector:" <<chanMapSortedChanID.size() << endl; // debug

  std::sort( chanMapSortedHardwareID.begin(),
	    chanMapSortedHardwareID.end(),
	    sortHardwareID() );

  // copy the channel map to a local copy which 
  // is then sortet according to padID
  chanMapSortedSoftwareID=*(dynamic_cast<LCCollectionVec *>(chanMapCol));
  chanMapSortedSoftwareID.setSubset();

  std::sort( chanMapSortedSoftwareID.begin(),
	     chanMapSortedSoftwareID.end(),
	     sortSoftwareID() );

  /*
  // debug: dump the sorted channel maps
  cout << "sortet by chanID:" << endl;
  for (LCCollectionVec::iterator it=chanMapSortedChanID.begin();
       it < chanMapSortedChanID.end(); it++)
  {
    cout << ADCChannelMapping(*it).getChannelID() << "\t"<<  ADCChannelMapping(*it).getPadID() << endl;
  }

  cout << "sortet by padID:" << endl;
  for (LCCollectionVec::iterator it=chanMapSortedPadID.begin();
       it < chanMapSortedPadID.end(); it++)
  {
    cout << ADCChannelMapping(*it).getChannelID() << "\t"<<  ADCChannelMapping(*it).getPadID() << endl;
  }
  */

  //  cout << "This is  MappingHandler:: setChannelMap" << endl; //debug
}

std::pair<int,int> MappingHandler::getHardwareID(int padID, int moduleID) const
{
  // The lower_bound performs a binary search
  // and returns an iterator to the lowest 
  // possible position where this value could be
  // inserted. If there already is an element with 
  // the specific value, it can be accessed by this
  // iterator.
  // As the padID is given, we use the collection
  // sorted in this order.
  LCCollectionVec::const_iterator it
    =std::lower_bound(chanMapSortedSoftwareID.begin(),
		      chanMapSortedSoftwareID.end(),
		      std::pair<int,int >(padID,moduleID),
		      cmpSoftwareID() );
  
  // Check whether the iterator realy accesses the correct
  // data. If the requested padID is not in the map
  // this is not the case. Throw an exception in this case.

  if ( (it ==  chanMapSortedSoftwareID.end()) ||
       (ADCChannelMapping(*it).getPadID() != padID) ||
       (ADCChannelMapping(*it).getModuleID() != moduleID))
  {
    std::stringstream message;
    message << "MappingHandler: Requested padID "<< padID
	    << " or moduleID "<< moduleID <<" not available in ADCChannelMap!";
    throw DataNotAvailableException(message.str() );
  }
  else
    return std::pair<int, int>( ADCChannelMapping(*it).getChannelID(),
				ADCChannelMapping(*it).getReadoutGroup() );
}

  std::pair<int, int> MappingHandler::getSoftwareID(int channelID, int readoutGroup) const
{
  // As the hardware IDs are given, we use the collection
  // sorted in this order.
  LCCollectionVec::const_iterator it
    =std::lower_bound(chanMapSortedHardwareID.begin(),
		      chanMapSortedHardwareID.end(),
		      std::pair<int, int>(channelID, readoutGroup),
		      cmpHardwareID() );
  
  // Check whether the iterator realy accesses the correct
  // data. If the requested padID is not in the map
  // this is not the case. Throw an exception in this case.

  if ( (it ==  chanMapSortedHardwareID.end()) ||
       (ADCChannelMapping(*it).getChannelID() != channelID) ||
       (ADCChannelMapping(*it).getReadoutGroup() != readoutGroup) )
    {
      std::stringstream message;
      message << "MappingHandler: Requested channelID "<< channelID
	      << " or readoutGroup "<< readoutGroup <<" not available in ADCChannelMap!";
      throw DataNotAvailableException(message.str() );

    }
  else
    return std::pair<int, int>( ADCChannelMapping(*it).getPadID(),
		      ADCChannelMapping(*it).getModuleID() );
}

}//namespace marlintpc
