//MarlinTPC
#include "TPCConditionsHandler.h"
#include "TPCCondDataExeptions.h"

//lccd
#include <lccd.h>
#include <lccd/LCConditionsMgr.hh>

// Marlin
#include <marlin/ConditionsProcessor.h>

// stl
#include <utility>
#include <map>
#include <iostream>
#include <string.h>

using namespace marlin;
using namespace tpcconddata;

namespace marlintpc
{

TPCConditionsHandler::TPCConditionsHandler( std::string collection_name ) {

	// register this handler with the lccd conditions processor
	ConditionsProcessor::registerChangeListener( this, collection_name );
}

TPCConditionsHandler::~TPCConditionsHandler() {

}


void TPCConditionsHandler::conditionsChanged( EVENT::LCCollection* col ) {

	// there is just one TPC conditions object for now, perhaps this should be extended one day in case more than one is needed. For example, there could be one TPCConditions object for each side of the TPC or one for each interactive region

	if( col->getNumberOfElements() > 1 ) {

		streamlog_out( WARNING ) << "There is more than one object in the TPC conditions collection. At this time only one object (the first one in the collection) is used. If more than one is needed then please update the TPCConditionsHandler class in MarlinTPC.\n";
	}

	_tpc_conditions = TPCConditions( col->getElementAt( 0 ) );

}


float TPCConditionsHandler::getDriftVelocity() const {

	return _tpc_conditions.getDriftVelocity();
}


float TPCConditionsHandler::getLongDiffusionCoef() const {

	return _tpc_conditions.getLongDiffusionCoef();
}


float TPCConditionsHandler::getLongDefocussing() const {

	return _tpc_conditions.getLongDefocussing();
}


float TPCConditionsHandler::getTransDiffusionCoef() const {

	return _tpc_conditions.getTransDiffusionCoef();
}


float TPCConditionsHandler::getTransDefocussing() const {

	return _tpc_conditions.getTransDefocussing();
}


float TPCConditionsHandler::getAmplification() const {

	return _tpc_conditions.getAmplification();
}

} // namespace marlintpc