#ifndef TimePixModeMapCreaterProcessor_h
#define TimePixModeMapCreaterProcessor_h 1

#include "marlin/DataSourceProcessor.h"
#include "lcio.h"
#include "IMPL/LCCollectionVec.h"
#include <fstream>

namespace marlin{
  
  /** Creates a mode map for a TimePix - there are four possible maps: MM1, MM2, TOT, TIME
   *  
   *
   *  <h4>Input - Prerequisites</h4>
   *  
   *
   *  <h4>Output</h4>
   *  \li A runheader
   *  \li 1 Event with TimePixModeMap (lcio::LCGenericObject)
   *
   * @param OutputCollectionName Name of the output collection, defaut \c TimePixModeMap
   * @param DetectorName Detector name for run header, default "TPC"
   * @param Description  Data descripton for run header, default "no description given"
   * @param SubDetectorName Sub-detector name for run header, default "TimePix"
   * @param Mode Operation mode of the chip: TOT, TIME, MM1 (mixed mode with CellID0 = 0: TIME), MM2 (mixed mode with CellID0 = 0: TOT)
   * @param NRow Number of rows on the chip, default 256
   * @param NColumn Number of columns on the chip, default 256
   * @param SetOutputTransient If not 0 the output collection is set transient
   * @param ChipIDs A list of Chip IDs for which a map is created
   *
   * @author Simone Zimmermann(University of Bonn) 
   */
  
  class TimePixModeMapCreaterProcessor : public DataSourceProcessor {
    
  public:

   TimePixModeMapCreaterProcessor() ;

    virtual TimePixModeMapCreaterProcessor*  newProcessor() ;


    /** Creates 1 event with LCGenericObject collections of TimePixModeMap
     *  calls all active processors' processEvent() and processRunHeader Method.
     *
     */
    virtual void readDataSource( int numEvents ) ;
    
    
    virtual void init() ;
    virtual void end() ;

    /** creates map of chip operated in MixedMode starting with TIMEPIX
     */    
    virtual void createMixedMode1Map( lcio::LCCollectionVec* modeMap );
    /** creates map of chip operated in MixedMode starting with TOT
     */    
    virtual void createMixedMode2Map( lcio::LCCollectionVec* modeMap );
    /** creates map of chip operated in TOT mode
     */    
    virtual void createTOTMap( lcio::LCCollectionVec* modeMap );
    /** creates map of chip operated in TIME mode
     */    
    virtual void createTIMEPIXMap( lcio::LCCollectionVec* modeMap );
    
  protected:
    
    std::string _outputColName;
    std::string _modeMapColName;

    bool _isFirstEvent;
    std::string _listName ;
    std::ifstream _infile;
    std::vector<int> _chipIDs;
    int _runNr;
    int _evtNr;
    int _nRow;
    int _nColumn;
    int _maxRecordN;
    std::string _detName;
    std::string _description;
    std::string _subDetName;
    std::string _mode;
    
/** give the status which is set to the transient flag of the output collection
 * it is an int instead of a boolean, because the processor parameter cannot be a boolean
 */
    int _outputIsTransient;
    
  };

 
} // end namespace marlin 

#endif
