#ifndef SWAP_XY_PROCESSOR_H
#define SWAP_XY_PROCESSOR_H

// C++
#include <string>

// LCIO
#include "lcio.h"

// Marlin
#include "marlin/Processor.h"

namespace marlintpc
{

    /** SwapXYProcessor swaps the x and the y coordinate of TrackerHits.
     * This processor MODIFIES the input collection, so you can only use it in the same
     * steering file which creates the collection.
     * If you try to run it on a collection read from a file you will get an exception.
     *
     *  <h4>Input - Prerequisites</h4>
     *  Collection with tracker hits
     *
     *  <h4>Output</h4>
     *  Collection with tracker hits
     *
     * @param InputCollectionName Name of input hits collection (default: TPCHits)
     */
    // @param OutputCollectionName (optional) Name of output hits collection (default: TPCSwappedHits)
    // @param SetOutputTransient If not 0 the output collection is set transient (default: 1)
    /**
     * @author Martin Killenberg, University of Bonn
     */
    class SwapXYProcessor: public marlin::Processor
    {

        public:
            virtual Processor*  newProcessor() { return new SwapXYProcessor ; }

            SwapXYProcessor() ;

//            virtual void init() ;

            virtual void processRunHeader( lcio::LCRunHeader* run ) ;

            virtual void processEvent( lcio::LCEvent * evt ) ;

//            virtual void check( lcio::LCEvent * evt ) ;

//            virtual void end() ;

        protected:
            std::string _inputColName ;///< The name of the input collection
//            std::string _outputColName ; ///< The name of the output collection

            /** give the status which is set to the transient flag of the output collection
             * it is an int instead of a boolean, because the processor parameter cannot be a boolean
             */
            int _outputIsTransient;
    } ;

}

#endif //SWAP_XY_PROCESSOR_H
