#include "DumpHitsInTracksProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerHit.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

DumpHitsInTracksProcessor aDumpHitsInTracksProcessor;
    
DumpHitsInTracksProcessor::DumpHitsInTracksProcessor(): Processor("DumpHitsInTracksProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACK,
				"InputTracks" , 
				"Name of the input tracks collection"  ,
				_inputColName ,
				std::string("TPCSeedTracks") ) ;
    // modify processor description
    _description = "DumpHitsInTracksProcessor: Dump coordinates of hits in tracks" ;
}
    
void DumpHitsInTracksProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    cout << "Event: " << evt->getEventNumber() << endl;

    // we need a LCCollectionVec to use iterators, as LCCollectios is o not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    if( InputCollection != 0 )
    {
	int trackNr=0;
	for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	    it < InputCollection->end(); it ++ )
	{
	    cout <<"\t TrackNumber "<<trackNr++<<endl;
	    Track * track = dynamic_cast<Track *>(*it) ;

	    TrackerHitVec hitVector = track->getTrackerHits();
	    
//	    cout << "nr of hits " << hitVector.size()<< endl;

	    int hitNr=0;
	    cout << "\t\t hit\t x \t y \t z" << endl;
	    for (TrackerHitVec::iterator hit = hitVector.begin();
		 hit <  hitVector.end(); hit ++)
	    {
		const double *pos = (*hit)->getPosition();
		cout << "\t\t "<<hitNr++<<"\t "<<pos[0]<<"\t "
		     <<pos[1]<<"\t "<<pos[2]<< endl;
	
	    }
        }

    }
}


}// namespace 
