#include "DumpResidualsProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/Track.h>
#include <EVENT/TrackerHit.h>
#include "IntersectionCalculator.h"

#include <marlin/Global.h>

#include "TrackFitterSimpleMinimizer.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

DumpResidualsProcessor aDumpResidualsProcessor;
    
    DumpResidualsProcessor::DumpResidualsProcessor(): Processor("DumpResidualsProcessor") , trackFitter(0)
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACK,
				"InputTracks" , 
				"Name of the input tracks collection"  ,
				_inputColName ,
				std::string("TPCSeedTracks") ) ;
    // modify processor description
    _description = "DumpResidualsProcessor: Dump residuals of hits in tracks" ;
}

void DumpResidualsProcessor::processRunHeader( EVENT::LCRunHeader* run )
{
    // delete the old track fitter (of none had been initialised it is 0, so it's save to call delete
    delete trackFitter;

    // instantiate new track fitter with parameters from run header
    trackFitter = new TrackFitterSimpleMinimizer();
//    trackFitter = new TrackFitterChiSquarePads(run,  & Global::GEAR->getTPCParameters() );
}
    
void DumpResidualsProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    cout << "Event: " << evt->getEventNumber() << endl;

    // we need a LCCollectionVec to use iterators, as LCCollectios is o not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    if( InputCollection != 0 )
    {
	int trackNr=0;
	for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	    it < InputCollection->end(); it ++ )
	{
	    cout <<"\t TrackNumber "<<trackNr++<<endl;
	    Track * track = dynamic_cast<Track *>(*it) ;

	    TrackerHitVec hitVector = track->getTrackerHits();
	    
//	    cout << "nr of hits " << hitVector.size()<< endl;

	    int hitNr=0;
	    cout << "\t\t hit\t x \t y \t z \t residual_x/r*phi" << endl;
	    for (TrackerHitVec::iterator hit = hitVector.begin();
		 hit <  hitVector.end(); hit ++)
	    {
		const double *pos = (*hit)->getPosition();
		cout << "\t\t "<<hitNr++<<"\t "<<pos[0]<<"\t "
		     <<pos[1]<<"\t "<<pos[2]<< "\t" ;

		try
		{
		    EVENT::DoubleVec residuals = trackFitter->calculateResiduals(*hit, track);
		    cout << residuals[0] << "\t" << residuals[1]  << std::endl;
		}
		catch ( IntersectionCalculator::NoIntersectionException &e )
		{
		    cout << e.what() << endl;
		}
	
	    }
        }

    }
}


}// namespace 
