#include "SwapXYProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <IMPL/TrackerHitImpl.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

SwapXYProcessor aSwapXYProcessor;
    
SwapXYProcessor::SwapXYProcessor(): Processor("SwapXYProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERHIT,
				"InputCollectionName" , 
				" Name of input hits collection (default: TPCHits)",
				_inputColName ,
				std::string("TPCHits") ) ;

//    registerOutputCollection(    LCIO::TRACKERHIT,
//				"OutputCollectionName" ,
//				"Name of output hits collection (default: TPCSwappedHits)",
//				_outputColName ,
//				std::string("TPCSwappedHits") ) ;
//    
    // modify processor description
    _description = "SwapXYProcessor swaps the x and the y coordinate of TrackerHits";
}

void SwapXYProcessor::processRunHeader( LCRunHeader* run) 
{
    run->parameters().setValue( _processorName + "_revision", "$Rev:$" );
}
    
void SwapXYProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data

    // we need a LCCollectionVec to use iterators, as LCCollectios is o not a std::vector
    LCCollectionVec* inputCollection=0;
    try{
	inputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }

    if( inputCollection != 0 )
    {
	// test if the collection is writeable
	try {
	    dynamic_cast<TrackerHitImpl*>(inputCollection->getElementAt(0))->setPosition(
		const_cast<double *>(dynamic_cast<TrackerHit*>(inputCollection->getElementAt(0))->getPosition()) );
	}
	catch (ReadOnlyException &e)
	{
	    cerr << "Error: collection is not writeable."<< endl;
	    cerr << "You can only use "<<_processorName<<" in the steering file which created the collection!" << endl;
	    throw lcio::ReadOnlyException(_inputColName+" not writeable");
	}

	for(LCCollectionVec::iterator it = inputCollection->begin() ; 
	    it < inputCollection->end(); it ++ )
	{
//	    cout <<"\t TrackNumber "<<trackNr++<<endl;
	    TrackerHitImpl * hit = dynamic_cast<TrackerHitImpl *>(*it) ;
	    
	    if (!hit) 
		throw DataNotAvailableException("Input collection does not contain TrackerHits");

	    double newPosition[3];
	    newPosition[0]= hit->getPosition()[1];
	    newPosition[1]= hit->getPosition()[0];
	    newPosition[2]= hit->getPosition()[2];

	    streamlog_out(DEBUG2) << hit->getPosition()[0] <<" , " 
				  << hit->getPosition()[1] <<" , " 
				  << hit->getPosition()[2] ;
	    hit->setPosition( newPosition );
	    streamlog_out(DEBUG2) << "  ->  ";
	    streamlog_out(DEBUG2) << hit->getPosition()[0] <<" , " 
				  << hit->getPosition()[1] <<" , " 
				  << hit->getPosition()[2] << std::endl;
        }

    }//( inputCollection != 0 )
}


}// namespace 
